use super::*;
use std::{
    path::Path,
};
use tokio::{
    fs::{
	OpenOptions,
    },
    prelude::*,
    stream::StreamExt,
};

mod tasklist;
#[macro_use]
mod progress;

//TODO: Create a module for temp files, pass the temp file to `perform` and do the regex fixing after `perform`

/// Download a loli async
pub async fn perform(url: impl AsRef<str>, path: impl AsRef<Path>, mut progress: progress::CommandSender) -> Result<(), error::Error>
{
    let url = url.as_ref();
    let path = path.as_ref();

    let task = url.to_owned(); //TODO: Real task name

    prog_send!(link progress.push_task(&task));
    
    let mut resp = reqwest::get(url).await?;
    let len = resp.content_length();

    
    let mut file = OpenOptions::new()
	.create(true)
	.truncate(true)
	.write(true)
	.read(true)
	.open(path).await?;
    
    prog_send!(progress.println(format!("req ok {:?}, file ok ({:?}), starting download", len, path)));
    let mut bytes = resp.bytes_stream();
    while let Some(buffer) = bytes.next().await {
	file.write(buffer?.as_ref()).await?;
	prog_send!(progress.bump());
    }

    //TODO: Decode `file`.

    prog_send!(progress.println(format!("done for {}", url)));
    prog_send!(progress.bump());

    prog_send!(progress.pop_task(task));

    Ok(())
}

pub async fn work(conf: config::Config) -> Result<(), Box<dyn std::error::Error>>
{
    let rating = conf.rating;
    let mut children = Vec::new();

    let prog = progress::AsyncProgressCounter::new("Initialising...");
    let mut prog_writer = prog.writer();

    let prog = prog.host();
    
    for path in conf.output.into_iter()
    {
	let url = url::parse(&rating);
	let mut prog = prog_writer.clone();
	children.push(tokio::task::spawn(async move {
	    
	    //println!("Starting download ({})...", url);
	    prog.println(format!("Starting download ({})...", url)).await.expect("fatal");
	    let path = match path {
		config::OutputType::File(file) => file,
		config::OutputType::Directory(dir) => {
		    //TODO: Implement downloading to temp and renaming to hash
		    unimplemented!();
		},
	    };
	    match perform(&url, &path, prog).await {
		Err(e) => panic!("Failed downloading {} -> {:?}: {}", url, path, e), //TODO: Make real error handler
		Ok(v) => v,
	    }
	}));
    }

    prog_send!(link prog_writer.println("Waiting for children..."));
    for child in children.into_iter()
    {
	match child.await {
	    Ok(v) => (),
	    Err(err) => {
		println!("Child failed: {}", err);
	    },
	}
    }
    
    prog_send!(try link prog_writer.kill());
    prog.await.expect("mpsc fatal");
    
    Ok(())
}
